<?php

namespace Drupal\document_ocr\Services;

use Google\ApiCore\ApiException;
use Google\Cloud\DocumentAI\V1\DocumentProcessorServiceClient;
use Google\Cloud\DocumentAI\V1\RawDocument;
use Google\Cloud\DocumentAI\V1\ProcessorVersion\State;

/**
 * Google Document AI service.
 */
class GoogleDocumentAi extends GoogleBase {

  /**
   * The processor parameters.
   *
   * @var array
   */
  protected $processor_params;

  /**
   * Get processor version name.
   */
  public function getVersionStateName($state) {
    return State::name($state);
  }

  /**
   * Initialize servie client.
   */
  public function serviceClient($location_id = 'us') {
    $params = [
      'credentials' => $this->credentials,
    ];
    if ($location_id != 'us') {
      $params['apiEndpoint'] = $location_id . '-documentai.googleapis.com';
    }
    $this->client = new DocumentProcessorServiceClient($params);
    return $this;
  }

  /**
   * Get a processor name.
   */
  public function getParams() {
    return $this->processor_params;
  }

  /**
   * Set a processor name.
   */
  public function setParams($processor_name) {
    $this->processor_params = $this->client->parseName($processor_name);
    return $this;
  }

  /**
   * Get processors.
   */
  public function getProcessors($location_id) {
    $processors = [];
    $cid = 'google_document_ai:processors:' . $this->getProjectId() . ':' . $location_id;
    if ($cache = $this->cacheBackend->get($cid)) {
      $processors = $cache->data;
    }
    else {
      $this->serviceClient($location_id);
      $formatted_parent = $this->client->locationName($this->getProjectId(), $location_id);
      try {
        $response = $this->client->listProcessors($formatted_parent);
        foreach ($response as $element) {
          $processors[$element->getName()] = $element;
        }
      }
      catch (ApiException $ex) {
        \Drupal::logger('document_ocr')->error($ex->getMessage());
      }
      $this->cacheBackend->set($cid, $processors, time() + $this->interval, [
        'google_document_ai:processors',
        'google_document_ai:processors:' . $location_id
      ]);
    }
    return $processors;
  }

  /**
   * Get processors versions.
   */
  public function getProcessorVersions() {
    $params = $this->getParams();
    $location_id = $params['location'];
    $processor_id = $params['processor'];
    $versions = [];
    $cid = 'google_document_ai:processor_version:' . $this->getProjectId() . ':' . $processor_id;
    if ($cache = $this->cacheBackend->get($cid)) {
      $versions = $cache->data;
    }
    else {
      $this->serviceClient($location_id);
      $formatted_parent = $this->client->processorName($params['project'], $location_id, $processor_id);
      try {
        $response = $this->client->listProcessorVersions($formatted_parent);
        foreach ($response as $element) {
          $versions[$element->getName()] = $element;
        }
      }
      catch (ApiException $ex) {
        \Drupal::logger('document_ocr')->error($ex->getMessage());
      }
      $this->cacheBackend->set($cid, $versions, time() + $this->interval, [
        'google_document_ai:processors',
        'google_document_ai:processor_version:' . $processor_id
      ]);
    }
    return $versions;
  }

  /**
   * Get locations.
   */
  public function getLocations() {
    $locations = [];
    $cid = 'google_document_ai:locations:' . $this->getProjectId();
    if ($cache = $this->cacheBackend->get($cid)) {
      $locations = $cache->data;
    }
    else {
      try {
        $response = $this->client->listLocations([
          'name' => 'projects/' . $this->getProjectId(),
        ]);
        foreach ($response as $element) {
          $locations[$element->getLocationId()] = $this->regionNames($element->getLocationId());
        }
      }
      catch (ApiException $ex) {
        \Drupal::logger('document_ocr')->error($ex->getMessage());
      }
      $this->cacheBackend->set($cid, $locations, time() + $this->interval, [
        'google_document_ai:locations',
      ]);
    }
    return $locations;
  }

  /**
   * Process document.
   */
  public function processDocument($file) {
    try {
      $params = $this->getParams();
      $this->serviceClient($params['location']);
      $documentPath = $this->fileSystem->realpath($file->getFileUri());

      $handle = fopen($documentPath, 'rb');
      $contents = fread($handle, filesize($documentPath));
      fclose($handle);

      $rawDocument = new RawDocument([
        'content' => $contents,
        'mime_type' => $file->getMimeType(),
      ]);

      if (!empty($params['processor_version'])) {
        $name = $this->client->processorVersionName($this->getProjectId(), $params['location'], $params['processor'], $params['processor_version']);
      }
      else {
        $name = $this->client->processorName($this->getProjectId(), $params['location'], $params['processor']);
      }
      $response = $this->client->processDocument($name, [
        'rawDocument' => $rawDocument
      ]);
      return $response->getDocument();
    }
    catch (ApiException $ex) {
      \Drupal::logger('document_ocr')->error($ex->getMessage());
    }
  }

  /**
   * Region names.
   */
  protected function regionNames($location_id) {
    $regions = [];
    $repository = \Drupal::service('document_ocr.documentai_regions_repository')->toArray();
    foreach ($repository as $item) {
      $regions[$item['region']] = $item['label'];
    }
    return !empty($regions[$location_id]) ? $regions[$location_id] : strtoupper($location_id);
  }

}
